@@ -15,7 +15,7 @@
 #' @examples \dontrun{
 #'  decode_gl("_p~iF~ps|U_ulLnnqC_mqNvxq`@@", precision = 5)
 #' }
-decode_gl <- function(polyline, precision=6, forceline=TRUE) {
+decode_gl <- function(polyline, precision = 6, forceline = TRUE) {
 
   latlngsets <- lapply(polyline, function(polyline, precision){
 
@@ -95,7 +95,9 @@ decode_gl <- function(polyline, precision=6, forceline=TRUE) {
 #'   exroutes <- viaroute(50, 0, 51, 1)
 #'   r <- viaroute2sldf(exroutes)
 #'   plot(r)
-#'   exroutes <- viaroute(viapoints=list(data.frame(x=c(-33.5,-33.6,-33.7),y=c(150,150.1,150.2))))
+#'   lngs <- c(-33.5,-33.6,-33.7)
+#'   lats <- c(150,150.1,150.2)
+#'   exroutes <- viaroute(viapoints = list(data.frame(x = lngs, y = lats)))
 #'   r <- viaroute2sldf(exroutes)
 #'   plot(r)
 #' }
@@ -354,6 +356,8 @@ viaroute <- function(startlat = NULL, startlng = NULL, endlat = NULL,
 #'
 #' @param osrmresult Single character string or character vector containing
 #' encoded json result(s) of OSRM routing queries.
+#' @param return_sf Boolean value if this function should return an sf object, if
+#' FALSE returns sp object (default FALSE).
 #' @export
 #' @examples \dontrun{
 #'  viaroute2sldf(
@@ -363,7 +367,7 @@ viaroute <- function(startlat = NULL, startlng = NULL, endlat = NULL,
 #'             endlng = 13.429290)
 #'  )
 #' }
-viaroute2sldf <- function(osrmresult) {
+viaroute2sldf <- function(osrmresult, return_sf = FALSE) {
 
   osrmsldf <- c()
   for (curroute in 1:length(osrmresult)) {
@@ -387,7 +391,8 @@ viaroute2sldf <- function(osrmresult) {
                              routecoords,
                              routename = osrmjson$route_name,
                              existrow = 0,
-                             routeid = ifelse(curroute == 1, 1, nrow(osrmsldf@data)+1))
+                             routeid = ifelse(curroute == 1, 1, nrow(osrmsldf@data)+1),
+                             return_sf = return_sf)
 
       if(is(osrmsldf, "SpatialLinesDataFrame") == FALSE) {
         osrmsldf <- osrmsldfalt
@@ -410,9 +415,14 @@ viaroute2sldf <- function(osrmresult) {
                              routecoords,
                              routename = osrmjson$alternative_names[[i]],
                              existrow = nrow(osrmsldf@data),
-                             routeid = ifelse(curroute == 1, 1, nrow(osrmsldf@data)+1)+i)
+                             routeid = ifelse(curroute == 1, 1, nrow(osrmsldf@data)+1)+i,
+                             return_sf = return_sf)
 
-          osrmsldf <- raster::bind(osrmsldf, osrmsldfalt)
+          if (return_sf == FALSE) {
+            osrmsldf <- raster::bind(osrmsldf, osrmsldfalt)
+          } else {
+            osrmsldf <- rbind(osrmsldf, osrmsldfalt)
+          }
 
           i <- i + 1
         }
@@ -429,11 +439,16 @@ viaroute2sldf <- function(osrmresult) {
           osrmjson$routes$legs[[i]]$steps[[j]]$geometry <- gsub('\\\\\\\\','\\\\',osrmjson$routes$legs[[i]]$steps[[j]]$geometry)
         }
       }
-      osrmsldfalt <- viaroute2sldf_instructv5(osrmjson, ifelse(curroute == 1, 1, nrow(osrmsldf@data)+1))
-      if(is(osrmsldf, "SpatialLinesDataFrame") == FALSE) {
+      osrmsldfalt <- viaroute2sldf_instructv5(osrmjson, ifelse(curroute == 1, 1, nrow(osrmsldf@data)+1), return_sf = return_sf)
+      if(is(osrmsldf, "SpatialLinesDataFrame") == FALSE & is(osrmsldf, "sf") == FALSE) {
         osrmsldf <- osrmsldfalt
       } else {
-        osrmsldf <- raster::bind(osrmsldf, osrmsldfalt)
+        if (return_sf == FALSE) {
+          osrmsldf <- raster::bind(osrmsldf, osrmsldfalt)
+        } else {
+          osrmsldf <- rbind(osrmsldf, osrmsldfalt)
+        }
+
       }
 
     }
@@ -444,7 +459,7 @@ viaroute2sldf <- function(osrmresult) {
 
 }
 
-viaroute2sldf_instruct <- function(routeinst, routesum, routecoords, routename = "", existrow = 0, routeid = 1) {
+viaroute2sldf_instruct <- function(routeinst, routesum, routecoords, routename = "", existrow = 0, routeid = 1, return_sf = FALSE) {
 
   if (class(routeinst) == "list") {
     routeinst <- unlist(routeinst, recursive = FALSE)
@@ -534,11 +549,15 @@ viaroute2sldf_instruct <- function(routeinst, routesum, routecoords, routename =
   osrmsldf@data$routesect <- 1:nrow(osrmsldf@data)
   osrmsldf@data$routeid <- routeid
 
+  if (return_sf == TRUE) {
+    osrmsldf <- sf::st_as_sf(osrmsldf)
+  }
+
   return(osrmsldf)
 
 }
 
-viaroute2sldf_instructv5 <- function(routeinst, startrouteid = 1) {
+viaroute2sldf_instructv5 <- function(routeinst, startrouteid = 1, return_sf = FALSE) {
 
   if (length(routeinst$routes$legs[[1]]$steps[[1]]$geometry) == 0) {
     osrmsldf <- sp::SpatialLinesDataFrame(
@@ -617,6 +636,10 @@ viaroute2sldf_instructv5 <- function(routeinst, startrouteid = 1) {
     osrmsldf@data <- osrmsldf@data[,-ncol(osrmsldf@data)]
   }
 
+  if (return_sf == TRUE) {
+    osrmsldf <- sf::st_as_sf(osrmsldf)
+  }
+
   return(osrmsldf)
 
 }
@@ -638,6 +661,8 @@ viaroute2sldf_instructv5 <- function(routeinst, startrouteid = 1) {
 #' @param profile OSRM profile to use (for API v5), defaults to "driving".
 #' @param protocol The protocol to use for the API (for v5), defaults to "v1".
 #' @param osrmurl Base URL of the OSRM service
+#' @param return_sf Boolean value if this function should return an sf object, if
+#' FALSE returns sp object (default FALSE).
 #' @export
 #' @examples \dontrun{
 #'  nearest_osm(
@@ -647,19 +672,25 @@ viaroute2sldf_instructv5 <- function(routeinst, startrouteid = 1) {
 #' }
 nearest_osm <- function(lat, lng, number = 1,
                         api = 5, profile="driving", protocol = "v1",
-                        osrmurl = "http://router.project-osrm.org"){
+                        osrmurl = "http://router.project-osrm.org",
+                        return_sf = FALSE){
   if (is(lat,"data.frame")) {
     lng = lat[,2]
     lat = lat[,1]
   }
   if (api == 4) {
     url = paste0(osrmurl, "/nearest?loc=", lat, ",", lng)
-    SpatialPointsDataFrame(coords = matrix(unlist(lapply(
+    thisspdf <- SpatialPointsDataFrame(coords = matrix(unlist(lapply(
                              url, function(x){
                                matrix(jsonlite::fromJSON(x)$mapped_coordinate, ncol=2)}
                              ),recursive = FALSE),ncol=2,byrow = TRUE),
                            data = data.frame(orig_lat = lat, orig_lng = lng),
                            proj4string = sp::CRS("+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs"))
+    if (return_sf == TRUE) {
+      sf::st_as_sf(thisspdf)
+    } else {
+      thisspdf
+    }
   } else {
     if(number < 1) {
       stop("number must be greater than 0")
@@ -669,7 +700,7 @@ nearest_osm <- function(lat, lng, number = 1,
       jsonlite::fromJSON(x)
     })
 
-    SpatialPointsDataFrame(coords = matrix(unlist(lapply(
+    thisspdf <- SpatialPointsDataFrame(coords = matrix(unlist(lapply(
       jsondata, function(x){
         matrix(x$waypoints$location[[1]], ncol=2)}
       ),recursive = FALSE),ncol=2,byrow = TRUE),
@@ -679,6 +710,13 @@ nearest_osm <- function(lat, lng, number = 1,
                      name = x$waypoints$name)
         }),.id = "locnum")),
         proj4string = sp::CRS("+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs"))
+
+    if (return_sf == TRUE) {
+      sf::st_as_sf(thisspdf)
+    } else {
+      thisspdf
+    }
+
   }
 }
 
@@ -694,6 +732,8 @@ nearest_osm <- function(lat, lng, number = 1,
 #' @param lng Numeric vector containing longitude coordinate for each
 #' coordinate to map.
 #' @param osrmurl Base URL of the OSRM service
+#' @param return_sf Boolean value if this function should return an sf object, if
+#' FALSE returns sp object (default FALSE).
 #' @export
 #' @examples \dontrun{
 #'  locate2spdf(
@@ -702,9 +742,13 @@ nearest_osm <- function(lat, lng, number = 1,
 #'  )
 #' }
 #'
-locate2spdf <- function(lat, lng = lng, osrmurl = "http://router.project-osrm.org") {
+locate2spdf <- function(lat, lng = lng, osrmurl = "http://router.project-osrm.org", return_sf = FALSE) {
 
-  return(getlocnear(lat = lat, lng = lng, osrmurl = osrmurl, "locate"))
+  if (return_sf == TRUE) {
+    return(sf::st_as_sf(getlocnear(lat = lat, lng = lng, osrmurl = osrmurl, "locate")))
+  } else {
+    return(getlocnear(lat = lat, lng = lng, osrmurl = osrmurl, "locate"))
+  }
 
 }
 
@@ -721,6 +765,8 @@ locate2spdf <- function(lat, lng = lng, osrmurl = "http://router.project-osrm.or
 #' @param lng Numeric vector containing longitude coordinate for each
 #' coordinate to map.
 #' @param osrmurl Base URL of the OSRM service
+#' @param return_sf Boolean value if this function should return an sf object, if
+#' FALSE returns sp object (default FALSE).
 #' @export
 #' @examples \dontrun{
 #'  nearest2spdf(
@@ -729,9 +775,13 @@ locate2spdf <- function(lat, lng = lng, osrmurl = "http://router.project-osrm.or
 #'  )
 #' }
 #'
-nearest2spdf <- function(lat, lng, osrmurl = "http://router.project-osrm.org") {
+nearest2spdf <- function(lat, lng, osrmurl = "http://router.project-osrm.org", return_sf = FALSE) {
 
-  return(getlocnear(lat = lat, lng = lng, osrmurl = osrmurl, "nearest"))
+  if (return_sf == TRUE) {
+    return(sf::st_as_sf(getlocnear(lat = lat, lng = lng, osrmurl = osrmurl, "nearest")))
+  } else {
+    return(getlocnear(lat = lat, lng = lng, osrmurl = osrmurl, "nearest"))
+  }
 
 }
 
@@ -876,3 +926,39 @@ table2matrix <- function(lat, lng, destlat = NA, destlng = NA,
   }
 
 }
+#' Plan a route with OSRM
+#'
+#' This is a wrapper around \code{viaroute} that returns a single route between A and B.
+#'
+#' @param singleline Should a single line be returned? Default is \code{TRUE}
+#' @param ... Arguments passed to viaroute()
+#' @inheritParams route_cyclestreet
+#' @inheritParams viaroute
+#' @export
+#' @examples
+#' \dontrun{
+#' from = c(-1.55, 53.80) # geo_code("leeds")
+#' to = c(-1.76, 53.80) # geo_code("bradford uk")
+#' r = route_osrm(from, to)
+#' plot(r)
+#' r_many <- line2route(flowlines_sf[2:9,], route_osrm)
+#' plot(cents)
+#' plot(r_many, add = TRUE)
+#' }
+route_osrm <- function(from, to, alt = FALSE, ..., singleline = TRUE) {
+
+  v <- viaroute(startlat = from[2], startlng = from[1], endlat = to[2], endlng = to[1], ...)
+  r <- viaroute2sldf(v)
+
+  if(!alt) {
+    r <- r[1:min(which(r$type == "arrive")), ]
+  }
+  if(singleline) {
+    r <- sp::SpatialLinesDataFrame(rgeos::gLineMerge(r),
+                                   data.frame(distance = sum(r$distance),
+                                              duration = sum(r$duration)))
+  }
+
+  return(r)
+
+}
