@@ -66,27 +66,17 @@
 #' @examples
 #'
 #' \dontrun{
-#' # Example from
-#' from = c(0.117950, 52.205302); to = c(0.131402, 52.221046)
+#' from = c(-1.55, 53.80) # geo_code("leeds")
+#' to = c(-1.76, 53.80) # geo_code("bradford uk")
 #' json_output = route_cyclestreet(from = from, to = to, plan = "quietest", save_raw = TRUE)
 #' str(json_output) # what does cyclestreets give you?
-#' names(json_output$marker$`@attributes`)
-#' json_output$marker$`@attributes`$start[1] # starting point
-#' json_output$marker$`@attributes`$finish[1] # end point
-#' json_output$marker$`@attributes`$speed[1] # assumed speed (km/hr)
-#' json_output$marker$`@attributes`$busynance # busyness of each section
-#' json_output$marker$`@attributes`$elevations # list of elevations
-#' # jsonlite::toJSON(json_output, pretty = TRUE) # complete json output (long!)
-#' # Plan the 'fastest' route between two points in Manchester
-#' rf_mcr <- route_cyclestreet(from = "M3 4EE", to = "M1 4BT", plan = "fastest")
-#' rf_mcr@data
-#' plot(rf_mcr)
-#' (rf_mcr$length / (1000 * 1.61)) / # distance in miles
-#'   (rf_mcr$time / (60 * 60)) # time in hours - average speed here: ~8mph
+#' rf_lb <- route_cyclestreet(from, to, plan = "fastest")
+#' rf_lb@data
+#' plot(rf_lb)
+#' (rf_lb$length / (1000 * 1.61)) / # distance in miles
+#'   (rf_lb$time / (60 * 60)) # time in hours - average speed here: ~8mph
 #' # Plan a 'balanced' route from Pedaller's Arms to the University of Leeds
-#' rb_pa <- route_cyclestreet("Pedaller's Arms, Leeds", "University of Leeds", "balanced")
-#' # A long distance route (max = 500 km) - lat/lon pairs in the UK
-#' route_cyclestreet(c(-2, 52), c(-1, 53), "fastest")
+#' rb_pa <- route_cyclestreet("Pedaller's Arms, Leeds", "University of Leeds, UK", "balanced")
 #' }
 #'
 route_cyclestreet <-
@@ -139,7 +129,7 @@ route_cyclestreet <-
     stop("Error: CycleStreets did not return a valid result")
   }
 
-  obj <- jsonlite::fromJSON(txt)
+  obj <- jsonlite::fromJSON(txt, simplifyDataFrame = TRUE)
 
   if (is.element("error", names(obj))) {
     stop(paste0("Error: ", obj$error))
@@ -232,7 +222,7 @@ route_cyclestreet <-
 #' @seealso route_cyclestreet
 #' @examples
 #' \dontrun{
-#' r <- route_graphhopper(from = "Leeds", to = "Dublin", vehicle = "bike")
+#' r <- route_graphhopper(from = "Leeds, UK", to = "Dublin, Ireland", vehicle = "bike")
 #' r@data
 #' plot(r)
 #' r <- route_graphhopper("New York", "Washington", vehicle = "foot")
