@@ -35,6 +35,16 @@
 #' @importFrom utils download.file tail unzip
 #' @importFrom maptools SpatialLinesMidPoints
 #' @importFrom rlang .data
-#'
+#' @importFrom sf st_cast st_geometry
 #' @useDynLib stplanr
-NULL
\ No newline at end of file
+NULL
+#' Pipe operator
+#'
+#' @name %>%
+#' @rdname pipe
+#' @keywords internal
+#' @export
+#' @importFrom dplyr %>%
+#' @usage lhs \%>\% rhs
+NULL
+utils::globalVariables(c(".", "n"))
\ No newline at end of file
